package GrabInfo::Plugin::_hosts ;
@ISA = qw(GrabInfo::Plugin) ;

use strict ;
use Socket ;
use Carp ;
use FindBin ;
use GrabInfo::VServer ;
use GrabInfo::Plugin::_hosts::_global ;
use GrabInfo::Plugin::_hosts::_addresses ;


sub run {
	my $this = shift ;

	# Get the list of all IP addresses assigned to the machine.
	my $lines = $this->run_cmd("$FindBin::Bin/ioctl") ;
	my @root_ips = () ;
	if ($lines){
		@root_ips = map {
			my $i = (split("\t", $_))[1] ;
			($i eq '127.0.0.1' ? () : $i) ;
		} @{$lines} ;
	}

	# Now we have the list of all the IP addresses assigned to this machine
	# What we have to do is figure out which ones are assigned to a vserver
	# and which ones a left to the root host.

	# Before processing the vservers, we need to make sure that
	# the kernel supports the syscalls we need to do
	my $vservers = undef ;
	if (GrabInfo::VServer::supported()){
		# Kernel supports vservers.
		# See if there are really some vservers here.
		$vservers = GrabInfo::VServer::find_vservers($this) ;
		if (! scalar(keys %{$vservers})){
			$vservers = undef ;
		}
	}

	if (defined($vservers)){
		# Now we have gathered some information on the vservers,
		# we need to go through the IP addresses for each vserver
		# and remove them from the root server's IP list.
		my %root_ips = map {($_ => 1)} @root_ips ;
		foreach my $v (values %{$vservers}){
			foreach my $i (@{$v->{iproot}}){
				delete $root_ips{$i} ;
			}
		}
		@root_ips = keys %root_ips ;

		# Now in some cases the vservers IP addresses can contain '0.0.0.0',
		# which means that this vserver is not restrained with respect to
		# the network. For such vservers, we must replace the 0.0.0.0 with
		# the list of IP addresses that is really assigned to the root server.
		# Here we assume the such vservers will not be permitted to interfere
		# with other vservers.
		foreach my $v (values %{$vservers}){
			foreach my $i (@{$v->{iproot}}){
				if ($i eq '0.0.0.0'){
					# For now we simply delete everything until we figure out 
					# how to deal with that.
					$v->{iproot} = [] ;
					last ;
				}
			}
		}
	}

	# Now the all the IP stuff is resolved, we proceed with the 
	# root host.
	$this->set_context_data("iproot", \@root_ips) ;
	my $rk = $this->{xml_tag}->create_kid('_root') ;
	$this->{xml_tag}->add_kid($rk) ;
	$rk->add_attr('_ip', join(' ', @root_ips)) ;
	$this->process_host($rk) ;

	if (defined($vservers)){
		my $vk = $this->{xml_tag}->create_kid('_vservers') ;
		$this->{xml_tag}->add_kid($vk) ;

		foreach my $v (values %{$vservers}){
			pipe(README, WRITEME) ;
			my $pid = fork() ;
			if (! defined($pid)){
				croak("Fork failed: $!") ;
			}
			elsif ($pid){
				close(WRITEME) ;
				my @lines = <README> ;
				close(README) ;
			
				my $ck = $vk->create_kid("dummy") ;
				$ck->{raw} = \@lines ;
				$vk->add_kid($ck) ;	

				waitpid($pid, 0) ;
			}
			else {
				# child
				close(README) ;

				# Add some data from to the context.
				$this->set_context_data("in_vserver", 1) ;
				$this->set_context_data("vserver_running", $v->{running}) ;

				my $name = $vk->validate_tag_name($v->{name}, 1) ;
				my $ck = $this->{xml_tag}->create_kid("_$v->{name}") ;
				$ck->{indent} = 0 ;
				$ck->add_attr('_profile', $v->{profile}) ;
				$ck->add_attr('_running', $v->{running}) ;
				$ck->add_attr('_ipv4root', join(' ', @{$v->{iproot}})) ;

				eval {
					my $ctx = GrabInfo::VServer::vserver_enter(
						$v->{name}, 
						$v->{context}, 
						@{$v->{iproot}}
					) ;
					$ck->add_attr('_s_context', $ctx) ;
					$this->set_context_data("iproot", $v->{iproot}) ;
					$this->process_host($ck) ;
				} ;
				if ($@){
					$ck->add_error($@) ;
				}

				$ck->dump(\*WRITEME) ;
				close(WRITEME) ;
				exit() ;
			}
		}
	}
}


sub snapshot {
}


sub process_host {
	my $this = shift ;
	my $parent = shift ;

	if (($this->get_context_data('in_vserver'))&&
		(! $this->get_context_data('vserver_running'))){
		return ;
	}

	eval {
		my $pig = new GrabInfo::Plugin::_hosts::_global("_global", $this->{context}, $parent->{indent} + 1) ;
		$pig->run() ;
		$parent->add_kid($pig->{xml_tag}) ;

		my $pia = new GrabInfo::Plugin::_hosts::_addresses("_addresses", $this->{context},  $parent->{indent} + 1) ;
		$pia->run() ;
		$parent->add_kid($pia->{xml_tag}) ;
	} ;
	if ($@){
		croak("$@\n") ;
	}
}



1 ;

