package GrabInfo::XMLTag ;


use strict ;
use Carp ;


sub new {
	my $class = shift ;
	my $name = shift ;
	my $indent = shift || 0 ;
	
	my $this = {
		indent => $indent,
		attrs => {},
		comment => '',
		kids => [],
		raw => undef,
	} ;
	bless($this, $class) ;

	$this->validate_tag_name($name) ;
	$this->{name} = $name ;

	return $this ;
}


sub validate_tag_name {
	my $this = shift ;
	my $name = shift ;
	my $fix = shift || 0 ;

	if ($fix){
		# You use fix mode when you have a random string that 
		# you want to use as a tag name. Here we forbid _ as the 
		# first char since it means that this tag is to ignored in the
   		# metric hierarchy. If you really want this you have to do it
		# yourselves afterwards.
		$name =~ s/[^\w-.]/_/g ;
		$name =~ s/^_+// ;
		return $name ;
	}
	else{
		if ($name !~ /^[\w-.]+$/){
			croak("Invalid XML tag/attribute name '$name'") ;
		}
	}

	return 1 ;
}


sub add_attr {
	my $this = shift ;
	my $name = shift ;
	my $value = shift ;

	$this->validate_tag_name($name) ;

	$this->{attrs}->{$name} = $this->escape($value) ;
}


sub create_kid {
	my $this = shift ;
	my $name = shift ;

	$this->validate_tag_name($name) ;

	return new GrabInfo::XMLTag($name, $this->{indent} + 1) ;
}


sub add_kid {
	my $this = shift ;
	my $kid = shift ;

	push @{$this->{kids}}, $kid ;
}


sub add_error {
	my $this = shift ;
	my $msg = shift ;

	my $t = new GrabInfo::XMLTag("_error", $this->{indent} + 1) ;
	$t->add_attr('_msg', $msg) ;
	$this->add_kid($t) ;
}


sub set_comment {
	my $this = shift ;
	my $msg = shift ;

	$this->{comment} = $msg ;
}


sub escape {
	my $this = shift ;
	my $text = shift ;

	$text =~ s/[&]/&amp;/g ;
	$text =~ s/[<]/&lt;/g ;
	$text =~ s/[>]/&gt;/g ;
	$text =~ s/["]/&quot;/g ;
	$text =~ s/[']/&apos;/g ;

	return $text ;
}


sub dump {
	my $this = shift ;
	my $handle = shift ;

	my $tab = "  " x $this->{indent} ;
	if ($this->{comment}){
		print $handle "$tab<!-- $this->{comment} -->\n" ;
	}

	if ($this->{raw}){
		foreach my $l (@{$this->{raw}}){
			chomp($l) ;
			print $handle "$tab$l\n" ;
		}
		return ;
	}

	my $attr_str = "" ;
	my @attrs = keys %{$this->{attrs}} ;
	if (scalar(@attrs)){
		$attr_str = " " . join(" ", 
			map {"$_='$this->{attrs}->{$_}'"} @attrs
		) ;	
	}

	if (! scalar(@{$this->{kids}})){
		print $handle "$tab<$this->{name}$attr_str/>\n" ;
	}
	else{
		print $handle "$tab<$this->{name}$attr_str>\n" ;
		foreach my $k (@{$this->{kids}}){
			$k->dump($handle) ;
		}
		print $handle "$tab</$this->{name}>\n" ;
	}
}


1 ;
						

